/*
 * Copyright (C) 2013, Boundary Devices <info@boundarydevices.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#undef MX6PAD
#undef MX6NAME

#ifdef FOR_DL_SOLO
#define MX6PAD(a) MX6DL_PAD_##a
#define MX6NAME(a) mx6dl_solo_##a
#else
#define MX6PAD(a) MX6Q_PAD_##a
#define MX6NAME(a) mx6q_##a
#endif

#define UART_PAD_CTRL  (PAD_CTL_PUS_100K_UP |			\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm |			\
	PAD_CTL_HYS | PAD_CTL_SRE_FAST)

#define USDHC_PAD_CTRL (PAD_CTL_PUS_47K_UP |			\
	PAD_CTL_SPEED_LOW | PAD_CTL_DSE_80ohm |			\
	PAD_CTL_SRE_FAST  | PAD_CTL_HYS)

#define ENET_PAD_CTRL  (PAD_CTL_PUS_100K_UP |			\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm | PAD_CTL_HYS)

#define SPI_PAD_CTRL (PAD_CTL_HYS | PAD_CTL_SPEED_MED |		\
	PAD_CTL_DSE_40ohm     | PAD_CTL_SRE_FAST)

#define BUTTON_PAD_CTRL (PAD_CTL_PUS_100K_UP |			\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm | PAD_CTL_HYS)

#define I2C_PAD_CTRL	(PAD_CTL_PUS_100K_UP |			\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm | PAD_CTL_HYS |	\
	PAD_CTL_ODE | PAD_CTL_SRE_FAST)

#define WEAK_PULLUP	(PAD_CTL_PUS_100K_UP |			\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm | PAD_CTL_HYS |	\
	PAD_CTL_SRE_SLOW)

#define WEAK_PULLDN	(PAD_CTL_PUS_100K_DOWN |		\
	PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm |			\
	PAD_CTL_HYS | PAD_CTL_SRE_SLOW)

#define OUTPUT_40OHM (PAD_CTL_SPEED_MED|PAD_CTL_DSE_40ohm)

static const iomux_v3_cfg_t MX6NAME(nitrogen6_max_pads)[] = {
	/* USB OTG */
	NEW_PAD_CTRL(MX6PAD(GPIO_1__USB_OTG_ID)	, WEAK_PULLUP),
	NEW_PAD_CTRL(MX6PAD(KEY_COL4__USB_OTG_OC), WEAK_PULLUP),
#define GP_USB_OTG_PWR	IMX_GPIO_NR(3, 22)		/* USB otg power */
	NEW_PAD_CTRL(MX6PAD(EIM_D22__GPIO3_IO22), OUTPUT_40OHM),

	/* USB host */
	NEW_PAD_CTRL(MX6PAD(EIM_D30__USB_H1_OC)	, WEAK_PULLUP),
#define GP_USB_HUB_RESET	IMX_GPIO_NR(7, 12)
	NEW_PAD_CTRL(MX6PAD(GPIO_17__GPIO7_IO12), OUTPUT_40OHM),

	/* UART1 */
	NEW_PAD_CTRL(MX6PAD(SD3_DAT7__UART1_TX_DATA), UART_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD3_DAT6__UART1_RX_DATA), UART_PAD_CTRL),

	/* UART2 */
	NEW_PAD_CTRL(MX6PAD(EIM_D26__UART2_TX_DATA), UART_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(EIM_D27__UART2_RX_DATA), UART_PAD_CTRL),

	/* UART3 for wl1271 */
	NEW_PAD_CTRL(MX6PAD(EIM_D24__UART3_TX_DATA), UART_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(EIM_D25__UART3_RX_DATA), UART_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(EIM_D23__UART3_CTS_B), UART_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(EIM_D31__UART3_RTS_B), UART_PAD_CTRL),


	/* UART5 - ISL3330IAZ rs485/rs232 selection */
	NEW_PAD_CTRL(MX6PAD(KEY_COL1__UART5_TX_DATA), UART_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(KEY_ROW1__UART5_RX_DATA), UART_PAD_CTRL),
#define GP_UART5_RX_EN		IMX_GPIO_NR(6, 10)
	NEW_PAD_CTRL(MX6PAD(NANDF_RB0__GPIO6_IO10), WEAK_PULLDN),	/* RS485 RX Enable */
#define GP_UART5_TX_EN		IMX_GPIO_NR(6, 7)
	NEW_PAD_CTRL(MX6PAD(NANDF_CLE__GPIO6_IO07), WEAK_PULLDN),	/* RS485 TX Enable */
#define GP_UART5_RS485_EN	IMX_GPIO_NR(2, 24)
	NEW_PAD_CTRL(MX6PAD(EIM_CS1__GPIO2_IO24), WEAK_PULLDN),		/* RS485/RS232 Select 2.5V */
#define GP_UART5_AON		IMX_GPIO_NR(6, 8)
	NEW_PAD_CTRL(MX6PAD(NANDF_ALE__GPIO6_IO08), WEAK_PULLDN),	/* ON - meaning depends on others */

	/* USDHC2 - TiWi wl1271 pads */
	NEW_PAD_CTRL(MX6PAD(SD2_CLK__SD2_CLK), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD2_CMD__SD2_CMD), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD2_DAT0__SD2_DATA0), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD2_DAT1__SD2_DATA1), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD2_DAT2__SD2_DATA2), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD2_DAT3__SD2_DATA3), USDHC_PAD_CTRL),
#define GP_WL12XX_WL_IRQ	IMX_GPIO_NR(6, 11)
	NEW_PAD_CTRL(MX6PAD(NANDF_CS0__GPIO6_IO11), WEAK_PULLDN),
#define GP_WL12XX_WL_ENABLE	IMX_GPIO_NR(6, 15)
	NEW_PAD_CTRL(MX6PAD(NANDF_CS2__GPIO6_IO15), OUTPUT_40OHM),
#define GP_WL12XX_BT_ENABLE	IMX_GPIO_NR(6, 16)
	NEW_PAD_CTRL(MX6PAD(NANDF_CS3__GPIO6_IO16), OUTPUT_40OHM),

	/* USDHC3 - sdcard */
	NEW_PAD_CTRL(MX6PAD(SD3_CLK__SD3_CLK), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD3_CMD__SD3_CMD), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD3_DAT0__SD3_DATA0), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD3_DAT1__SD3_DATA1), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD3_DAT2__SD3_DATA2), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD3_DAT3__SD3_DATA3), USDHC_PAD_CTRL),
#define GP_SD3_CD		IMX_GPIO_NR(7, 0)
	NEW_PAD_CTRL(MX6PAD(SD3_DAT5__GPIO7_IO00), WEAK_PULLUP),
#define GP_SD3_POWER_SEL	IMX_GPIO_NR(6, 14)
	NEW_PAD_CTRL(MX6PAD(NANDF_CS1__SD3_VSELECT), OUTPUT_40OHM),

	/* USDHC4 - emmc */
	NEW_PAD_CTRL(MX6PAD(SD4_CLK__SD4_CLK), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_CMD__SD4_CMD), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_DAT0__SD4_DATA0), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_DAT1__SD4_DATA1), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_DAT2__SD4_DATA2), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_DAT3__SD4_DATA3), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_DAT4__SD4_DATA4), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_DAT5__SD4_DATA5), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_DAT6__SD4_DATA6), USDHC_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(SD4_DAT7__SD4_DATA7), USDHC_PAD_CTRL),
#define GP_EMMC_RESET	IMX_GPIO_NR(2, 6)
	NEW_PAD_CTRL(MX6PAD(NANDF_D6__GPIO2_IO06), OUTPUT_40OHM), /* EMMC_RESET */

	/* ENET pads that don't change for PHY reset */
	NEW_PAD_CTRL(MX6PAD(ENET_MDIO__ENET_MDIO), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(ENET_MDC__ENET_MDC), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_TXC__RGMII_TXC), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_TD0__RGMII_TD0), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_TD1__RGMII_TD1), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_TD2__RGMII_TD2), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_TD3__RGMII_TD3), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_TX_CTL__RGMII_TX_CTL), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(ENET_REF_CLK__ENET_TX_CLK), ENET_PAD_CTRL),
	/* pin 42 PHY nRST */
#define GP_ENET_PHY_RESET	IMX_GPIO_NR(1, 27)
	NEW_PAD_CTRL(MX6PAD(ENET_RXD0__GPIO1_IO27), OUTPUT_40OHM),

	/* ECSPI1 pads */
#define GP_ECSPI1_SS1	IMX_GPIO_NR(3, 19)
	NEW_PAD_CTRL(MX6PAD(EIM_D19__GPIO3_IO19), SPI_PAD_CTRL), /* SS1 */
	NEW_PAD_CTRL(MX6PAD(EIM_D17__ECSPI1_MISO), SPI_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(EIM_D18__ECSPI1_MOSI), SPI_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(EIM_D16__ECSPI1_SCLK), SPI_PAD_CTRL),

	/* Button pad assignments for J14 */
#define GP_BUTTON_MENU		IMX_GPIO_NR(2, 1)
	NEW_PAD_CTRL(MX6PAD(NANDF_D1__GPIO2_IO01), BUTTON_PAD_CTRL),
#define GP_BUTTON_BACK		IMX_GPIO_NR(2, 2)
	NEW_PAD_CTRL(MX6PAD(NANDF_D2__GPIO2_IO02), BUTTON_PAD_CTRL),
	/* Labeled Search (mapped to Power under Android) */
#define GP_BUTTON_SEARCH	IMX_GPIO_NR(2, 3)
	NEW_PAD_CTRL(MX6PAD(NANDF_D3__GPIO2_IO03), BUTTON_PAD_CTRL),
#define GP_BUTTON_HOME		IMX_GPIO_NR(2, 4)
	NEW_PAD_CTRL(MX6PAD(NANDF_D4__GPIO2_IO04), BUTTON_PAD_CTRL),
#define GP_BUTTON_VOLDOWN	IMX_GPIO_NR(7, 1)
	NEW_PAD_CTRL(MX6PAD(SD3_DAT4__GPIO7_IO01), BUTTON_PAD_CTRL),
#define GP_BUTTON_VOLUP		IMX_GPIO_NR(7, 13)
	NEW_PAD_CTRL(MX6PAD(GPIO_18__GPIO7_IO13), BUTTON_PAD_CTRL),

	/* Backlight on RGB connector: J15, pin 37 */
#define GP_RGB_BACKLIGHT	IMX_GPIO_NR(1, 21)
	NEW_PAD_CTRL(MX6PAD(SD1_DAT3__GPIO1_IO21), OUTPUT_40OHM),
#define GP_LVDS_BACKLIGHT	IMX_GPIO_NR(1, 18)
	/* Backlight on LVDS connector: J6, pin 20 */
	NEW_PAD_CTRL(MX6PAD(SD1_CMD__GPIO1_IO18), OUTPUT_40OHM),

	/* Camera controls */
#define GP_OV5642_CSI0_PWRDN	IMX_GPIO_NR(3, 29)
	NEW_PAD_CTRL(MX6PAD(EIM_D29__GPIO3_IO29), OUTPUT_40OHM),
#define GP_OV5642_CSI0_RESET	IMX_GPIO_NR(1, 4)
	NEW_PAD_CTRL(MX6PAD(GPIO_4__GPIO1_IO04), OUTPUT_40OHM),

#define GP_OV5640_CSI1_PWRDN	IMX_GPIO_NR(3, 13)
	MX6PAD(EIM_DA13__GPIO3_IO13),		/* pin 32 - Power */
#define GP_OV5640_CSI1_RESET	IMX_GPIO_NR(3, 14)
	MX6PAD(EIM_DA14__GPIO3_IO14),		/* pin 36 - Reset */

#define GP_OV5640_MIPI_PWRDN	IMX_GPIO_NR(6, 9)
	NEW_PAD_CTRL(MX6PAD(NANDF_WP_B__GPIO6_IO09), OUTPUT_40OHM),
#define GP_OV5640_MIPI_RESET	IMX_GPIO_NR(2, 5)
	NEW_PAD_CTRL(MX6PAD(NANDF_D5__GPIO2_IO05), OUTPUT_40OHM),

	/* PCIe */
#define GP_PCIE_RESET		IMX_GPIO_NR(6, 31)
	NEW_PAD_CTRL(MX6PAD(EIM_BCLK__GPIO6_IO31), OUTPUT_40OHM),

	/* rtc */
#define GP_RTC_RV4162_IRQ	IMX_GPIO_NR(4, 6)
	NEW_PAD_CTRL(MX6PAD(KEY_COL0__GPIO4_IO06), WEAK_PULLUP),

	/* i2c enables */
#define GP_I2C_EN_OV5642	IMX_GPIO_NR(3, 20)
	NEW_PAD_CTRL(MX6PAD(EIM_D20__GPIO3_IO20), OUTPUT_40OHM),		/* ov5642 camera i2c enable */
#define GP_I2C_EN_OV5640_MIPI	IMX_GPIO_NR(3, 20)
	NEW_PAD_CTRL(MX6PAD(KEY_ROW4__GPIO4_IO15), OUTPUT_40OHM),		/* ov5640_mipi camera i2c enable */
#define GP_I2C_EN_PCIE		IMX_GPIO_NR(3, 20)
	NEW_PAD_CTRL(MX6PAD(EIM_OE__GPIO2_IO25), OUTPUT_40OHM),			/* pcie i2c enable */

	/* Amplifier Mute */
	NEW_PAD_CTRL(MX6PAD(ENET_TXD1__GPIO1_IO29),WEAK_PULLDN),
};

static const iomux_v3_cfg_t MX6NAME(enet_pads1)[] = {
	/* pin 35 - 1 (PHY_AD2) on reset */
	NEW_PAD_CTRL(MX6PAD(RGMII_RXC__GPIO6_IO30), OUTPUT_40OHM),
	/* pin 32 - 1 - (MODE0) all */
	NEW_PAD_CTRL(MX6PAD(RGMII_RD0__GPIO6_IO25), OUTPUT_40OHM),
	/* pin 31 - 1 - (MODE1) all */
	NEW_PAD_CTRL(MX6PAD(RGMII_RD1__GPIO6_IO27), OUTPUT_40OHM),
	/* pin 28 - 1 - (MODE2) all */
	NEW_PAD_CTRL(MX6PAD(RGMII_RD2__GPIO6_IO28), OUTPUT_40OHM),
	/* pin 27 - 1 - (MODE3) all */
	NEW_PAD_CTRL(MX6PAD(RGMII_RD3__GPIO6_IO29), OUTPUT_40OHM),
	/* pin 33 - 1 - (CLK125_EN) 125Mhz clockout enabled */
	NEW_PAD_CTRL(MX6PAD(RGMII_RX_CTL__GPIO6_IO24), OUTPUT_40OHM),
};

static const iomux_v3_cfg_t MX6NAME(enet_pads2)[] = {
	NEW_PAD_CTRL(MX6PAD(RGMII_RXC__RGMII_RXC), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_RD0__RGMII_RD0), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_RD1__RGMII_RD1), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_RD2__RGMII_RD2), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_RD3__RGMII_RD3), ENET_PAD_CTRL),
	NEW_PAD_CTRL(MX6PAD(RGMII_RX_CTL__RGMII_RX_CTL), ENET_PAD_CTRL),
};


static const iomux_v3_cfg_t MX6NAME(rgb_pads)[] = {
	MX6PAD(DI0_DISP_CLK__IPU1_DI0_DISP_CLK),
	MX6PAD(DI0_PIN15__IPU1_DI0_PIN15),
	MX6PAD(DI0_PIN2__IPU1_DI0_PIN02),
	MX6PAD(DI0_PIN3__IPU1_DI0_PIN03),
	MX6PAD(DI0_PIN4__GPIO4_IO20),
	MX6PAD(DISP0_DAT0__IPU1_DISP0_DATA00),
	MX6PAD(DISP0_DAT1__IPU1_DISP0_DATA01),
	MX6PAD(DISP0_DAT2__IPU1_DISP0_DATA02),
	MX6PAD(DISP0_DAT3__IPU1_DISP0_DATA03),
	MX6PAD(DISP0_DAT4__IPU1_DISP0_DATA04),
	MX6PAD(DISP0_DAT5__IPU1_DISP0_DATA05),
	MX6PAD(DISP0_DAT6__IPU1_DISP0_DATA06),
	MX6PAD(DISP0_DAT7__IPU1_DISP0_DATA07),
	MX6PAD(DISP0_DAT8__IPU1_DISP0_DATA08),
	MX6PAD(DISP0_DAT9__IPU1_DISP0_DATA09),
	MX6PAD(DISP0_DAT10__IPU1_DISP0_DATA10),
	MX6PAD(DISP0_DAT11__IPU1_DISP0_DATA11),
	MX6PAD(DISP0_DAT12__IPU1_DISP0_DATA12),
	MX6PAD(DISP0_DAT13__IPU1_DISP0_DATA13),
	MX6PAD(DISP0_DAT14__IPU1_DISP0_DATA14),
	MX6PAD(DISP0_DAT15__IPU1_DISP0_DATA15),
	MX6PAD(DISP0_DAT16__IPU1_DISP0_DATA16),
	MX6PAD(DISP0_DAT17__IPU1_DISP0_DATA17),
	MX6PAD(DISP0_DAT18__IPU1_DISP0_DATA18),
	MX6PAD(DISP0_DAT19__IPU1_DISP0_DATA19),
	MX6PAD(DISP0_DAT20__IPU1_DISP0_DATA20),
	MX6PAD(DISP0_DAT21__IPU1_DISP0_DATA21),
	MX6PAD(DISP0_DAT22__IPU1_DISP0_DATA22),
	MX6PAD(DISP0_DAT23__IPU1_DISP0_DATA23),
};

/*
 *
 */
#define PC I2C_PAD_CTRL

struct i2c_pads_info MX6NAME(i2c_pad_info)[] = {
{
	/* I2C1, SGTL5000 */
	.scl = {
		.i2c_mode = NEW_PAD_CTRL(MX6PAD(EIM_D21__I2C1_SCL), PC),
		.gpio_mode = NEW_PAD_CTRL(MX6PAD(EIM_D21__GPIO3_IO21), PC),
		.gp = IMX_GPIO_NR(3, 21)
	},
	.sda = {
		.i2c_mode = NEW_PAD_CTRL(MX6PAD(EIM_D28__I2C1_SDA), PC),
		.gpio_mode = NEW_PAD_CTRL(MX6PAD(EIM_D28__GPIO3_IO28), PC),
		.gp = IMX_GPIO_NR(3, 28)
	}
}, {
	/* I2C2 Camera, MIPI */
	.scl = {
		.i2c_mode = NEW_PAD_CTRL(MX6PAD(KEY_COL3__I2C2_SCL), PC),
		.gpio_mode = NEW_PAD_CTRL(MX6PAD(KEY_COL3__GPIO4_IO12), PC),
		.gp = IMX_GPIO_NR(4, 12)
	},
	.sda = {
		.i2c_mode = NEW_PAD_CTRL(MX6PAD(KEY_ROW3__I2C2_SDA), PC),
		.gpio_mode = NEW_PAD_CTRL(MX6PAD(KEY_ROW3__GPIO4_IO13), PC),
		.gp = IMX_GPIO_NR(4, 13)
	}
}, {
	/* I2C3, J15 - RGB connector */
	.scl = {
		.i2c_mode = NEW_PAD_CTRL(MX6PAD(GPIO_5__I2C3_SCL), PC),
		.gpio_mode = NEW_PAD_CTRL(MX6PAD(GPIO_5__GPIO1_IO05), PC),
		.gp = IMX_GPIO_NR(1, 5)
	},
	.sda = {
		.i2c_mode = NEW_PAD_CTRL(MX6PAD(GPIO_16__I2C3_SDA), PC),
		.gpio_mode = NEW_PAD_CTRL(MX6PAD(GPIO_16__GPIO7_IO11), PC),
		.gp = IMX_GPIO_NR(7, 11)
	}
}
};
